@@ -19,6 +19,8 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
+#include "libavutil/channel_layout.h"
+#include "libavutil/opt.h"
 #include "libavcodec/flac.h"
 #include "avformat.h"
 #include "flacenc.h"
@@ -26,6 +28,14 @@
 #include "libavcodec/bytestream.h"
 
 
+typedef struct FlacMuxerContext {
+    const AVClass *class;
+    int write_header;
+
+    /* updated streaminfo sent by the encoder at the end */
+    uint8_t *streaminfo;
+} FlacMuxerContext;
+
 static int flac_write_block_padding(AVIOContext *pb, unsigned int n_padding_bytes,
                                     int last_block)
 {
@@ -42,12 +52,12 @@ static int flac_write_block_comment(AVIOContext *pb, AVDictionary **m,
                                     int last_block, int bitexact)
 {
     const char *vendor = bitexact ? "Libav" : LIBAVFORMAT_IDENT;
-    unsigned int len, count;
+    unsigned int len;
     uint8_t *p, *p0;
 
     ff_metadata_conv(m, ff_vorbiscomment_metadata_conv, NULL);
 
-    len = ff_vorbiscomment_length(*m, vendor, &count);
+    len = ff_vorbiscomment_length(*m, vendor);
     p0 = av_malloc(len+4);
     if (!p0)
         return AVERROR(ENOMEM);
@@ -55,7 +65,7 @@ static int flac_write_block_comment(AVIOContext *pb, AVDictionary **m,
 
     bytestream_put_byte(&p, last_block ? 0x84 : 0x04);
     bytestream_put_be24(&p, len);
-    ff_vorbiscomment_write(&p, m, vendor, count);
+    ff_vorbiscomment_write(&p, m, vendor);
 
     avio_write(pb, p0, len+4);
     av_freep(&p0);
@@ -68,13 +78,35 @@ static int flac_write_header(struct AVFormatContext *s)
 {
     int ret;
     AVCodecContext *codec = s->streams[0]->codec;
+    FlacMuxerContext *c   = s->priv_data;
 
-    ret = ff_flac_write_header(s->pb, codec, 0);
+    if (!c->write_header)
+        return 0;
+
+    ret = ff_flac_write_header(s->pb, codec->extradata,
+                               codec->extradata_size, 0);
     if (ret)
         return ret;
 
+    /* add the channel layout tag */
+    if (codec->channel_layout &&
+        !(codec->channel_layout & ~0x3ffffULL) &&
+        !ff_flac_is_native_layout(codec->channel_layout)) {
+        AVDictionaryEntry *chmask = av_dict_get(s->metadata, "WAVEFORMATEXTENSIBLE_CHANNEL_MASK",
+                                                NULL, 0);
+
+        if (chmask) {
+            av_log(s, AV_LOG_WARNING, "A WAVEFORMATEXTENSIBLE_CHANNEL_MASK is "
+                   "already present, this muxer will not overwrite it.\n");
+        } else {
+            uint8_t buf[32];
+            snprintf(buf, sizeof(buf), "0x%"PRIx64, codec->channel_layout);
+            av_dict_set(&s->metadata, "WAVEFORMATEXTENSIBLE_CHANNEL_MASK", buf, 0);
+        }
+    }
+
     ret = flac_write_block_comment(s->pb, &s->metadata, 0,
-                                   codec->flags & CODEC_FLAG_BITEXACT);
+                                   s->flags & AVFMT_FLAG_BITEXACT);
     if (ret)
         return ret;
 
@@ -90,12 +122,13 @@ static int flac_write_header(struct AVFormatContext *s)
 static int flac_write_trailer(struct AVFormatContext *s)
 {
     AVIOContext *pb = s->pb;
-    uint8_t *streaminfo;
-    enum FLACExtradataFormat format;
     int64_t file_size;
+    FlacMuxerContext *c = s->priv_data;
+    uint8_t *streaminfo = c->streaminfo ? c->streaminfo :
+                                          s->streams[0]->codec->extradata;
 
-    if (!avpriv_flac_is_extradata_valid(s->streams[0]->codec, &format, &streaminfo))
-        return -1;
+    if (!c->write_header || !streaminfo)
+        return 0;
 
     if (pb->seekable) {
         /* rewrite the STREAMINFO header block data */
@@ -107,25 +140,58 @@ static int flac_write_trailer(struct AVFormatContext *s)
     } else {
         av_log(s, AV_LOG_WARNING, "unable to rewrite FLAC header.\n");
     }
+
+    av_freep(&c->streaminfo);
+
     return 0;
 }
 
 static int flac_write_packet(struct AVFormatContext *s, AVPacket *pkt)
 {
-    avio_write(s->pb, pkt->data, pkt->size);
-    avio_flush(s->pb);
+    FlacMuxerContext *c = s->priv_data;
+    uint8_t *streaminfo;
+    int streaminfo_size;
+
+    /* check for updated streaminfo */
+    streaminfo = av_packet_get_side_data(pkt, AV_PKT_DATA_NEW_EXTRADATA,
+                                         &streaminfo_size);
+    if (streaminfo && streaminfo_size == FLAC_STREAMINFO_SIZE) {
+        av_freep(&c->streaminfo);
+
+        c->streaminfo = av_malloc(FLAC_STREAMINFO_SIZE);
+        if (!c->streaminfo)
+            return AVERROR(ENOMEM);
+        memcpy(c->streaminfo, streaminfo, FLAC_STREAMINFO_SIZE);
+    }
+
+    if (pkt->size)
+        avio_write(s->pb, pkt->data, pkt->size);
     return 0;
 }
 
+static const AVOption flacenc_options[] = {
+    { "write_header", "Write the file header", offsetof(FlacMuxerContext, write_header), AV_OPT_TYPE_INT, {.i64 = 1}, 0, 1, AV_OPT_FLAG_ENCODING_PARAM },
+    { NULL },
+};
+
+static const AVClass flac_muxer_class = {
+    .class_name = "flac muxer",
+    .item_name  = av_default_item_name,
+    .option     = flacenc_options,
+    .version    = LIBAVUTIL_VERSION_INT,
+};
+
 AVOutputFormat ff_flac_muxer = {
     .name              = "flac",
     .long_name         = NULL_IF_CONFIG_SMALL("raw FLAC"),
+    .priv_data_size    = sizeof(FlacMuxerContext),
     .mime_type         = "audio/x-flac",
     .extensions        = "flac",
-    .audio_codec       = CODEC_ID_FLAC,
-    .video_codec       = CODEC_ID_NONE,
+    .audio_codec       = AV_CODEC_ID_FLAC,
+    .video_codec       = AV_CODEC_ID_NONE,
     .write_header      = flac_write_header,
     .write_packet      = flac_write_packet,
     .write_trailer     = flac_write_trailer,
-    .flags= AVFMT_NOTIMESTAMPS,
+    .flags             = AVFMT_NOTIMESTAMPS,
+    .priv_class        = &flac_muxer_class,
 };
