@@ -24,11 +24,11 @@
 
 AVFifoBuffer *av_fifo_alloc(unsigned int size)
 {
-    AVFifoBuffer *f= av_mallocz(sizeof(AVFifoBuffer));
-    if(!f)
+    AVFifoBuffer *f = av_mallocz(sizeof(AVFifoBuffer));
+    if (!f)
         return NULL;
     f->buffer = av_malloc(size);
-    f->end = f->buffer + size;
+    f->end    = f->buffer + size;
     av_fifo_reset(f);
     if (!f->buffer)
         av_freep(&f);
@@ -37,7 +37,7 @@ AVFifoBuffer *av_fifo_alloc(unsigned int size)
 
 void av_fifo_free(AVFifoBuffer *f)
 {
-    if(f){
+    if (f) {
         av_free(f->buffer);
         av_free(f);
     }
@@ -59,12 +59,13 @@ int av_fifo_space(AVFifoBuffer *f)
     return f->end - f->buffer - av_fifo_size(f);
 }
 
-int av_fifo_realloc2(AVFifoBuffer *f, unsigned int new_size) {
-    unsigned int old_size= f->end - f->buffer;
+int av_fifo_realloc2(AVFifoBuffer *f, unsigned int new_size)
+{
+    unsigned int old_size = f->end - f->buffer;
 
-    if(old_size < new_size){
-        int len= av_fifo_size(f);
-        AVFifoBuffer *f2= av_fifo_alloc(new_size);
+    if (old_size < new_size) {
+        int len          = av_fifo_size(f);
+        AVFifoBuffer *f2 = av_fifo_alloc(new_size);
 
         if (!f2)
             return -1;
@@ -72,45 +73,48 @@ int av_fifo_realloc2(AVFifoBuffer *f, unsigned int new_size) {
         f2->wptr += len;
         f2->wndx += len;
         av_free(f->buffer);
-        *f= *f2;
+        *f = *f2;
         av_free(f2);
     }
     return 0;
 }
 
-// src must NOT be const as it can be a context for func that may need updating (like a pointer or byte counter)
-int av_fifo_generic_write(AVFifoBuffer *f, void *src, int size, int (*func)(void*, void*, int))
+/* src must NOT be const as it can be a context for func that may need
+ * updating (like a pointer or byte counter) */
+int av_fifo_generic_write(AVFifoBuffer *f, void *src, int size,
+                          int (*func)(void *, void *, int))
 {
     int total = size;
     do {
         int len = FFMIN(f->end - f->wptr, size);
-        if(func) {
-            if(func(src, f->wptr, len) <= 0)
+        if (func) {
+            if (func(src, f->wptr, len) <= 0)
                 break;
         } else {
             memcpy(f->wptr, src, len);
-            src = (uint8_t*)src + len;
+            src = (uint8_t *)src + len;
         }
 // Write memory barrier needed for SMP here in theory
         f->wptr += len;
         if (f->wptr >= f->end)
             f->wptr = f->buffer;
         f->wndx += len;
-        size -= len;
+        size    -= len;
     } while (size > 0);
     return total - size;
 }
 
-
-int av_fifo_generic_read(AVFifoBuffer *f, void *dest, int buf_size, void (*func)(void*, void*, int))
+int av_fifo_generic_read(AVFifoBuffer *f, void *dest, int buf_size,
+                         void (*func)(void *, void *, int))
 {
 // Read memory barrier needed for SMP here in theory
     do {
         int len = FFMIN(f->end - f->rptr, buf_size);
-        if(func) func(dest, f->rptr, len);
-        else{
+        if (func)
+            func(dest, f->rptr, len);
+        else {
             memcpy(dest, f->rptr, len);
-            dest = (uint8_t*)dest + len;
+            dest = (uint8_t *)dest + len;
         }
 // memory barrier needed for SMP here in theory
         av_fifo_drain(f, len);
@@ -141,9 +145,9 @@ int main(void)
         av_fifo_generic_write(fifo, &i, sizeof(int), NULL);
 
     /* peek at FIFO */
-    n = av_fifo_size(fifo)/sizeof(int);
-    for (i = -n+1; i < n; i++) {
-        int *v = (int *)av_fifo_peek2(fifo, i*sizeof(int));
+    n = av_fifo_size(fifo) / sizeof(int);
+    for (i = -n + 1; i < n; i++) {
+        int *v = (int *)av_fifo_peek2(fifo, i * sizeof(int));
         printf("%d: %d\n", i, *v);
     }
     printf("\n");
